'use strict';

var gulp = require('gulp');
var webpack = require('webpack');

var gutil = require('gulp-util');
var htmlone = require('gulp-htmlone');

var webpackConf = require('./webpack.config');

var src = process.cwd() + '/src';
var build = process.cwd() + '/build';

// clean build
gulp.task('clean', function() {
  var clean = require('gulp-clean');
  return gulp.src(build, {read: true}).pipe(clean());
});

// webpack打包
gulp.task('webpack',function(done) {
  webpack(webpackConf, function(err, stats) {
    if(err) throw new gutil.PluginError('webpack', err);
    gutil.log('[webpack]', stats.toString({colors: true}));
    done();
  });
});

gulp.task('htm' , function(){
  return gulp.src('./src/pages/*.htm').pipe(htmlone({
              removeSelector: '[will-remove]', // selector 配置，源文档中符合这个selector的标签将自动移除
              keepliveSelector: '[keeplive]',  // 不要combo的selector配置，源文档中符合这个selector的将保持原状
              destDir: './build',  // dest 目录配置，一般情况下需要和 gulp.dest 目录保持一致，用于修正相对路径资源
              coimport: true,  // 是否需要把 @import 的资源合并
              cssminify: true, // 是否需要压缩css
              jsminify: false  // 是否需要压缩js
          })).pipe(gulp.dest('./build'));
});

// 打包
gulp.task('build',['clean','webpack'])

// webpack-dev-server本地服务器
gulp.task('server', function(done) {
  var WebpackDevServer = require('webpack-dev-server');

  webpackConf.devtool = 'source-map';

  webpackConf.entry = webpackConf.hotDebugEntry;
  webpackConf.plugins = webpackConf.hotDebugPlugins;

  var compiler = webpack(webpackConf);
  var devSvr = new WebpackDevServer(compiler, {
    hot: true,
    publicPath: webpackConf.output.publicPath,
  });

  devSvr.listen(8080, '127.0.0.1', function(err) {
    if(err) throw new gutil.PluginError('webpack-dev-server', err);

    gutil.log('[webpack-dev-server]','http://127.0.0.1:8080/');
  });
});


