#!/usr/bin/env node 
var fs = require('fs');
var path = require('path');
var exec = require('child_process').exec; 
var chalk = require('chalk');

// console.log((process.env._));
// console.log(fs.readlinkSync(process.env._));
var sourcePath = path.join(path.dirname(process.env._) , path.dirname(fs.readlinkSync(process.env._))) + '/';

//console.log(sourcePath);

var help = function(){
    console.log('the reactjs user guide');
    console.log(chalk.green('reactjs : ') + 'show the help of reactjs');
    console.log(chalk.green('reactjs -h : ') + 'show the help of reactjs');
    console.log(chalk.green('reactjs -i <projectName> : ') + 'init a project named <projectName>');
    console.log(chalk.green('reactjs -p <pageName>: ') + 'create one page name <pageName>');
    console.log(chalk.green('reactjs -c <componentName>: ') + 'create one component name <componentName>');
    console.log(chalk.green('reactjs <componentName>: ') + 'create one component name <componentName>');
}

function copyFile(sourceFile , desFile , p , fullPath){
    var str = '';
    if(fullPath){
        str = fs.readFileSync(sourceFile).toString();
    }
    else{
        str = fs.readFileSync(sourcePath + sourceFile).toString();
    }
    
    var writerStream = fs.createWriteStream(desFile);
    for(var key in p){
        str = str.replace(new RegExp("{{"+key+"}}" , 'g') , p[key]);
    }
    writerStream.write(str.toString(),'UTF8');
    writerStream.end();
    //console.log("创建文件：" + desFile);
}




var initProject = function(param){
    fs.readdir('.' , function(err , files){
        if(err) {
            console.log(chalk.red.bold(err));
            return;
        } else {
            var flag = true;
            files.forEach(function(file){
                if('gulpfile.js,package.json.webpack.config.js,src'.indexOf(file) > -1){
                    flag = false;
                }
            });
            if(!flag){
                console.log('can not init \'' + param.name + '\' , ' + chalk.red.bold(' please be sure this is an empty directory') );
                return ;
            }
            copyFile('base/project/package.json' , './package.json' , {name : param.name});
            copyFile('base/project/gitignore' , './.gitignore' , {});
            copyFile('base/project/gulpfile.js' , './gulpfile.js' , {});
            copyFile('base/project/webpack.config.js' , './webpack.config.js' , {});
            fs.mkdirSync('src');
            fs.mkdirSync('src/pages');
            fs.mkdirSync('src/components');
            copyFile('base/page/common.less' , './src/pages/common.less' , {});
            var commond = 'cp -rf ' + sourcePath + 'base/project/utils' + ' ./src/';
            exec(commond);
            console.log( chalk.cyan('init \'' + param.name + '\' successed') );
        }
    });
}

var createPage = function(param){
    var path = process.env.PWD;
    if(fs.existsSync('src/pages/' + param.name + '.js')){
        return console.log('can not create the page \''+ param.name +'\' , because ' + chalk.red.bold('the page \''+ param.name +'\' has exised') );
    }

    if(fs.existsSync('src/pages')){
        copyFile( "base/page/base-page.htm" , './src/pages/' + param.name  + '.htm' , param);
        copyFile( "base/page/base-page.js" , './src/pages/' + param.name  + '.js' , param);
        copyFile( "base/page/base-page.less" , './src/pages/' + param.name  + '.less' , param);
        copyFile( "./webpack.config.js" , './webpack.config.js' , {
            hotDebugEntry : "{{hotDebugEntry}}\t\n\t\t'" + param.name + "' : ['webpack/hot/dev-server', './src/pages/" + param.name + ".js'] ," ,
            entry : "{{entry}}\t\n\t\t'"  + param.name + "' : ['./src/pages/"+param.name+".js'], "
        } , true);
        console.log( chalk.cyan('create \'' + param.name + '\' page successed') );
    }
    else {
        console.log('can not find the directory \'src/pages\' , please use the \'reactjs -p {name}\' code ' + chalk.red.bold(' in an react inited project') );
    }
}

var createComponent = function(param){
    if(fs.existsSync('src/components/' + param.path)){
        return console.error('can not create the component \''+ param.name +'\' , because ' + chalk.red.bold('the page \''+ param.name +'\' has exised') );
    }
    if(fs.existsSync('src/components')){
        fs.mkdirSync('src/components/' + param.path);
        copyFile("base/component/BaseComponent.js" , './src/components/' + param.path + '/' + param.name  + '.js' , param);
        copyFile("base/component/BaseComponent.less" , './src/components/' + param.path + '/' + param.name  + '.less' , param);
        copyFile("base/component/BaseComponent.mock.js" , './src/components/' + param.path + '/' + param.name  + '.mock.js' , param);
        console.log( chalk.cyan('create \'' + param.name + '\' component successed') );
    }
    else {
        console.log('can not find the directory \'src/components\' , please use the \'reactjs -p {name}\' code ' + chalk.red.bold(' in an react inited project') );
    }
}


var formatParam = function(obj){
    var param = {type : '-h'};
    var error = false;
    if(obj.length == 1){
        param = obj[0].indexOf('-') == 0 ? {type : obj[0]} : {type : '-c' , name : obj[0]};
    }
    else if(obj.length == 2){
        param = obj[0].indexOf('-') == 0 ? {type : obj[0] , name : obj[1]} : {type : obj[1] , name : obj[0]};
    }
    if('-h-i-p-c'.indexOf(param.type) == -1){
        error = true;
    }  
    if('-i-p-c'.indexOf(param.type) != -1 && !param.name){
        error = true;
    }
    
    //处理名字
    if(param.type == '-c'){
        var name = param.name;
        var path = name.replace(/[A-Z]/g , function(c , i){
            if(i == 0){
                return c.toLocaleLowerCase();
            }
            return '-' + c.toLocaleLowerCase();
        });
        name =('-' +  name).replace(/\-[a-z,A-Z]/g , function(c , i){
            return  c.substring(1).toLocaleUpperCase();
        });
        param.path = path;
        param.name = name;
    }
    if(param.type == '-i' || param.type == '-p'){
        param.name = param.name.replace(/[A-Z]/g , function(c , i){
            if(i == 0){
                return c.toLocaleLowerCase();
            }
            return '-' + c.toLocaleLowerCase();
        });
    }
    if(error){
        console.log(chalk.red.bold('parameter error .') + ' Please use the reactjs -h to view how to use the command');
        param = false;
    }
    return param;
}


var run = function(obj){
    var param = formatParam(obj);
    if(param.type == '-i'){
        initProject(param);
    }
    if(param.type == '-p'){
        createPage(param);
    }
    if(param.type == '-c'){
        createComponent(param);
    }
    if(param.type == '-h'){
        help();
    }
}

run(process.argv.slice(2 , 4));



// console.log(chalk.black('test a color!haha!!! do you think this is beautiful'));
// console.log(chalk.red('test a color!haha!!! do you think this is beautiful'));
// console.log(chalk.green('test a color!haha!!! do you think this is beautiful'));
// console.log(chalk.yellow('test a color!haha!!! do you think this is beautiful'));
// console.log(chalk.blue('test a color!haha!!! do you think this is beautiful'));
// console.log(chalk.magenta('test a color!haha!!! do you think this is beautiful'));
// console.log(chalk.cyan('test a color!haha!!! do you think this is beautiful'));
// console.log(chalk.white('test a color!haha!!! do you think this is beautiful'));
// console.log(chalk.gray('test a color!haha!!! do you think this is beautiful'));







